"""
Example for oolin - linear oofun

For the given input vector z it returns dot(f, z) + c 
(by default c=0)

Note! For oolin NaN * 0 had been implemented as 0.
For example oolin([0, 0, 1]) with x = [nan, 10, 15] will yield 15
"""
from openopt import *
from numpy import *
v0 = oovar('v0', [-4, -15]) # 2nd arg (if provided) is start value
v1 = oovar('v1', [1, 2], ub=[0.1, 0.2]) # ub is upper bound (optional)
v2 = oovar('v2', (3, 4), lb=[-2, 0], ub=[-1.5, 1]) # lb is lower bound (optional)

f0 = oofun(lambda z: z[0]**2 + z[0]  + 2*z[1]**2 , input = v0, name='f0')
f1 = oofun(lambda z: (z-1).sum() ** 2, input = v1, name='f1')
f2 = oofun(lambda z: z.sum() ** 2, input = v2, name='f2')


"""
f3 returns z0 + 2 * z1 + 3 * z2
this is same to 
f3 = oofun(lambda z0, z1, z2: z0 + 2*z1 + 3*z2 , input = [f0, f1, f2], d = lambda *args: [1, 2, 3])
"""
f3 = oolin([1, 2, 3], input=[f0, f1, f2])


"""
f4 returns 15 * f0 + 8 * f3 + 80 * f2 + 15
this is same to
f4 = oofun(lambda z0, z1, z2: 15*z0 + 8*z1 + 80*z2 + 15 , input = [f0, f4, f2], d = lambda *args: [15, 8, 80])
"""
f4 = oolin([15, 8, 80], 15, input=[f0, f3, f2]) 

"""
f5 returns matrix_C * vector_input + vector_d
We will use it as 3 non-linear inequalities C * input + d <= 0 
(input is non-linear, hence the constraints are non-linear):
z1 + 2*z2 + 4*z3 - 10 <= 0
4*z1 + 5*z2 + 8*z3 - 20 <= 0
7*z1 + 8*z2 + 10*z3 - 30 <= 0
Of course, we could use f5 somewhere as a part of (recursive) calculations for obtaining objFunc
"""
C = mat('1 2 4; 4 5 8; 7 8 10') # numpy.array or array-like object is OK as well
d = [-10, -20, -30] # numpy.array or array-like object is OK as well
f5 = oolin(C, d, input=[f2, f1, f0])

# This one will be automatically rendered into  *linear* constraints Ax <= b
# -v0[0] - 2*v0[1] - 3*v1[0] - 4*v1[1] + 1 <= 0
# -1.01*v0[0] - 2.01*v0[1] - 2*v1[0] -3*v1[1] + 1.01 <= 0
# note that b will be -1, not 1, since oolin(C, d) is Cx + d, not Cx <= d
f6 = oolin([[-1, -2, -3, -4], [-1.01, -2.01, -2, -3]], [1, 1.01], input = (v0, v1))

# objective function:
F = oolin([1, 1, 1], 15, input = [f2, f3, f4]) # returns f2 + f3 + f4 + 15

# assign prob:
p = NLP(F, c=[f5, f6]) # Note - it already has some lb <= x <= ub constraints rased from v1, v2
 
# solve:
r = p.solve('ralg')
#r = p.solve('ipopt')
#r = p.solve('scipy_slsqp')

print 'solution:', r.xf
print 'optim value:', r.ff
"""
solution: {'v0': array([-0.26245289,  0.23636635]), 'v1': array([ 0.09999883,  0.20000055]), 'v2': array([-1.49999927,  0.99999872])}
optim value: 107.056098957
"""
