"""
This example illustrates a great speedup that can be achieved
via using oofun and oovar vs "classic" style,
even for unconstrained functions,
provided the solver deals with at least 1st derivatives,
so scipy_cobyla, goldenSection, scipy_fminbound, scipy_powell
or GLP solvers are inappropriate

The speedup is due to changes in derivatives numerical approximation approach:
instead of handling whole dF/dx = d(g(f))/dx
we find dg/df, df/dx and
dF/dx = dg/df * df/dx

Here's example of unconstrained problem, but constrained ones can be used as well
for NLP, NSP, NLSP, NLLSP classes

Concider the NL problem
g(f(x)) -> min
g is costly and g derivative are not available
f(x) = (x[0]-0)^2 + (x[1]-1)^2 + ... + (x[N-1]-(N-1))^2
see below for definition of g
here I have chosed g: R -> R for the sake of simplicity,
but R^m -> R^k can be handled as well
"""
def CostlyFunction(z):
    counter['g'] += 1
    r = 0
    for k in xrange(1, K+2):
        r += z ** (1 / k**1.5)
    return r

def f(z):
    counter['f'] += 1
    return ((z-aN)**2).sum()

solver = 'ralg'# try also scipy_cg, scipy_ncg, ralg, algencan etc
N, K = 100, 500
ftol, xtol, gtol = 1e-6, 1e-6, 1e-6
iprint = 50

from openopt import NLP,  oofun,  oovar
from numpy import arange, zeros
aN = arange(N)

"""                      1: using oovar & oofun                      """
counter = {'f':0, 'g':0}
v = oovar('v', size = N) # start value will be zeros(N)
ff = oofun(f, input = v)
g = oofun(CostlyFunction, input = ff)
p = NLP(g, maxIter=1e4, iprint=iprint, ftol=ftol, xtol=xtol, gtol=gtol)
print 'using oofun:'
r = p.solve(solver)
print 'evals f:', counter['f'], '  evals of costly func g:', counter['g']
"""                               2: classic                                  """
counter = {'f':0, 'g':0}
g = CostlyFunction
p2 = NLP(lambda x: g(f(x)), x0=zeros(N), maxIter=1e4, ftol=ftol, xtol=xtol, gtol=gtol, iprint=iprint)
print '\nwithout oofun:'
r2 = p2.solve(solver)
print 'evals f:', counter['f'], '  evals of costly func g:', counter['g']


"""
using oofun:
-----------------------------------------------------
solver: ralg   problem: unnamed   goal: minimum
 iter    objFunVal   
    0  3.290e+05 
   50  4.888e+02 
  100  4.844e+02 
  150  4.797e+02 
  151  4.797e+02 
istop:  3 (|| X[k] - X[k-1] || < xtol)
Solver:   Time Elapsed = 5.23 	CPU Time Elapsed = 5.22
objFunValue: 479.7321
evals f: 19216   evals of costly func g: 802

without oofun:
-----------------------------------------------------
solver: ralg   problem: unnamed   goal: minimum
 iter    objFunVal   
    0  3.290e+05 
   50  4.902e+02 
  100  4.855e+02 
  150  4.830e+02 
  200  4.813e+02 
  214  4.810e+02 
istop:  3 (|| X[k] - X[k-1] || < xtol)
Solver:   Time Elapsed = 29.64 	CPU Time Elapsed = 28.42
objFunValue: 481.02874
evals f: 26049   evals of costly func g: 26049
"""
